@@ -2,19 +2,19 @@ import argparse
 import os
 import shutil
 import subprocess
-from typing import Union
+from typing import Optional
 
 from docker.errors import ImageNotFound
 
 from cpk import CPKProject
-from cpk.utils.docker import get_client, DOCKER_INFO, \
-    get_endpoint_architecture, pull_image
-from cpk.utils.misc import sanitize_hostname, human_size, configure_binfmt
+from cpk.utils.misc import configure_binfmt
+from .endpoint import CLIEndpointInfoCommand
 from .info import CLIInfoCommand
 from .. import AbstractCLICommand
 from ..logger import cpklogger
 from ...exceptions import NotACPKProjectException
-from ...types import CPKFileMappingTrigger
+from ...types import CPKFileMappingTrigger, Machine, Arguments
+from ...utils.cli import check_git_status
 
 DEFAULT_NETWORK_MODE = "bridge"
 SUPPORTED_SUBCOMMANDS = [
@@ -28,7 +28,8 @@ class CLIRunCommand(AbstractCLICommand):
     KEY = 'run'
 
     @staticmethod
-    def parser(parent: Union[None, argparse.ArgumentParser] = None) -> argparse.ArgumentParser:
+    def parser(parent: Optional[argparse.ArgumentParser] = None,
+               args: Optional[Arguments] = None) -> argparse.ArgumentParser:
         parser = argparse.ArgumentParser(parents=[parent])
         parser.add_argument(
             "-n",
@@ -54,12 +55,12 @@ class CLIRunCommand(AbstractCLICommand):
             action="store_true",
             help="Whether to force pull the image of the project",
         )
-        parser.add_argument(
-            "--plain",
-            default=False,
-            action="store_true",
-            help="Whether to run the image without default module configuration",
-        )
+        # parser.add_argument(
+        #     "--plain",
+        #     default=False,
+        #     action="store_true",
+        #     help="Whether to run the image without default module configuration",
+        # )
         parser.add_argument(
             "--no-multiarch",
             default=False,
@@ -142,91 +143,70 @@ class CLIRunCommand(AbstractCLICommand):
             nargs="*",
             default=[]
         )
-        parser.add_argument(
-            "subcommand",
-            nargs="?",
-            default=None,
-            choices=SUPPORTED_SUBCOMMANDS,
-            help="(Optional) Subcommand to execute"
-        )
+        # parser.add_argument(
+        #     "subcommand",
+        #     nargs="?",
+        #     default=None,
+        #     choices=SUPPORTED_SUBCOMMANDS,
+        #     help="(Optional) Subcommand to execute"
+        # )
         return parser
 
     @staticmethod
-    def execute(parsed: argparse.Namespace) -> bool:
-        parsed.workdir = os.path.abspath(parsed.workdir)
-
+    def execute(machine: Machine, parsed: argparse.Namespace) -> bool:
         # get project
         project = CPKProject(parsed.workdir, parsed=parsed)
 
         # show info about project
-        CLIInfoCommand.execute(parsed)
+        CLIInfoCommand.execute(machine, parsed)
 
-        # check if the git HEAD is detached
-        if project.is_detached():
-            cpklogger.error(
-                "The repository HEAD is detached. Create a branch or check one out "
-                "before continuing. Aborting."
-            )
-            return False
-
-        # check if the index is clean
+        # check git workspace status
         mount_source = parsed.mount is True or isinstance(parsed.mount, str)
-        if project.is_dirty() and mount_source:
-            cpklogger.warning("Your index is not clean (some files are not committed).")
-            cpklogger.warning("If you know what you are doing, use --force (-f) to force.")
-            if not parsed.force:
-                return False
-            cpklogger.warning("Forced!")
-
-        # sanitize hostname
-        if parsed.machine is not None:
-            parsed.machine = sanitize_hostname(parsed.machine)
-
-        # create docker client
-        docker = get_client(parsed.machine)
+        proceed = check_git_status(project, parsed, must_be_clean=mount_source)
+        if not proceed:
+            return False
 
         # get info about docker endpoint
-        cpklogger.info("Retrieving info about Docker endpoint...")
-        epoint = docker.info()
-        if "ServerErrors" in epoint:
-            cpklogger.error("\n".join(epoint["ServerErrors"]))
-            return False
-        epoint["MemTotal"] = human_size(epoint["MemTotal"])
-        cpklogger.print(DOCKER_INFO.format(**epoint))
+        CLIEndpointInfoCommand.execute(machine, parsed)
 
-        # pick the right architecture if not set
+        # pick right value of `arch` given endpoint
+        machine_arch = machine.get_architecture()
         if parsed.arch is None:
-            parsed.arch = get_endpoint_architecture(parsed.machine)
-            cpklogger.info(f"Target architecture automatically set to {parsed.arch}.")
+            cpklogger.info("Parameter `arch` not given, will resolve it from the endpoint.")
+            parsed.arch = machine_arch
+            cpklogger.info(f"Parameter `arch` automatically set to `{parsed.arch}`.")
+
+        # create docker client
+        docker = machine.get_client()
 
         # create defaults
         image = project.image(parsed.arch)
         parsed.name = parsed.name or f"cpk-run-{project.name}"
 
         # subcommand "attach"
-        if parsed.subcommand == "attach":
-            cpklogger.info(f"Attempting to attach to container '{parsed.name}'...")
-            # run
-            _run_cmd(
-                [
-                    "docker",
-                    "-H=%s" % parsed.machine,
-                    "exec",
-                    "-it",
-                    parsed.name,
-                    "/entrypoint.sh",
-                    "bash",
-                ],
-                suppress_errors=True,
-            )
-            return True
+        # TODO: this will not work with Machine created from env, the host will be None
+        # if parsed.subcommand == "attach":
+        #     cpklogger.info(f"Attempting to attach to container '{parsed.name}'...")
+        #     # run
+        #     _run_cmd(
+        #         [
+        #             "docker",
+        #             "-H=%s" % machine.host,
+        #             "exec",
+        #             "-it",
+        #             parsed.name,
+        #             "/entrypoint.sh",
+        #             "bash",
+        #         ],
+        #         suppress_errors=True,
+        #     )
+        #     return True
 
         # print info about multiarch
-        msg = "Running an image for {} on {}.".format(parsed.arch, epoint["Architecture"])
-        cpklogger.info(msg)
+        cpklogger.info("Running an image for {} on {}.".format(parsed.arch, machine_arch))
         # - register bin_fmt in the target machine (if needed)
         if not parsed.no_multiarch:
-            configure_binfmt(parsed.arch, docker, cpklogger)
+            configure_binfmt(machine_arch, parsed.arch, docker, cpklogger)
 
         # ---
         # TODO: this is wrong, x-docker should either be a Python package that gives us the
@@ -241,11 +221,6 @@ class CLIRunCommand(AbstractCLICommand):
             raise ValueError('Docker runtime binary "{}" not found!'.format(parsed.runtime))
         # ---
 
-        # pick the right architecture if not set
-        if parsed.arch is None:
-            parsed.arch = get_endpoint_architecture(parsed.machine)
-            cpklogger.info(f"Target architecture automatically set to {parsed.arch}.")
-
         # create the module configuration
         module_configuration_args = [
             f"--net={parsed.network_mode}"
@@ -297,7 +272,7 @@ class CLIRunCommand(AbstractCLICommand):
                 cpklogger.info("Found an image with the same name. Use --force-pull to pull again")
             else:
                 cpklogger.info('Pulling image "%s"...' % image)
-                pull_image(image, docker)
+                machine.pull_image(image)
 
         # cmd option
         if parsed.cmd and parsed.launcher:
@@ -316,8 +291,9 @@ class CLIRunCommand(AbstractCLICommand):
 
         # endpoint arguments
         docker_epoint_args = []
-        if parsed.machine is not None:
-            docker_epoint_args += ["-H", parsed.machine]
+        # TODO: this should be using the Python SDK for Docker, not Docker CLI
+        if machine.base_url is not None:
+            docker_epoint_args += ["-H", machine.base_url]
 
         # docker arguments
         if not parsed.docker_args:
@@ -352,7 +328,8 @@ class CLIRunCommand(AbstractCLICommand):
             cpklogger.debug(f"Command exited with exit code [{exitcode}].")
 
 
-def _run_cmd(cmd, get_output=False, print_output=False, suppress_errors=False, shell=False, return_exitcode=False):
+def _run_cmd(cmd, get_output=False, print_output=False, suppress_errors=False, shell=False,
+             return_exitcode=False):
     if shell and isinstance(cmd, (list, tuple)):
         cmd = " ".join([str(s) for s in cmd])
     cpklogger.debug("$ %s" % cmd)
