@@ -1,18 +1,25 @@
+import abc
 import copy
-import os
+import dataclasses
 import json
+import logging
+import os
 from enum import Enum
+from shutil import rmtree
+from typing import List, Dict, Optional
 
 import jsonschema
+from docker import DockerClient
 
+from cpk.utils.progress_bar import ProgressBar
 from .constants import CANONICAL_ARCH
 from .exceptions import \
     NotACPKProjectException, \
     InvalidCPKTemplateFile, InvalidCPKTemplate, \
-    CPKTemplateSchemaNotSupported
+    CPKTemplateSchemaNotSupported, CPKException
 from .schemas import get_template_schema
-import dataclasses
-from typing import List, Dict, Optional
+
+Arguments = List[str]
 
 
 @dataclasses.dataclass
@@ -273,3 +280,146 @@ class CPKTemplateInfo:
             raise InvalidCPKTemplateFile(path, "Missing field: `schema`")
         # ---
         return CPKTemplateInfo.from_template_dict(data)
+
+
+class Machine(abc.ABC):
+    type: str
+
+    def __init__(self, name: str, base_url: Optional[str] = None,
+                 configuration: Optional[Dict] = None):
+        self._name = name
+        self._base_url = base_url
+        self._configuration = configuration or {}
+        # cache
+        self._arch = None
+
+    @property
+    def name(self) -> str:
+        return self._name
+
+    @property
+    def config_path(self) -> str:
+        from cpk import cpkconfig
+        return os.path.join(cpkconfig.path, "machines", self.name)
+
+    @property
+    def base_url(self) -> Optional[str]:
+        return self._base_url
+
+    @abc.abstractmethod
+    def get_client(self) -> DockerClient:
+        raise NotImplementedError("Method 'get_client' needs to be implemented by child class")
+
+    def __enter__(self):
+        pass
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
+    def save(self, logger: Optional[logging.Logger] = None):
+        from . import cpkconfig
+        path = os.path.join(cpkconfig.path, "machines", self.name)
+        # make directory
+        if logger:
+            logger.debug(f"Creating directory '{path}'")
+        os.makedirs(path, mode=0o700, exist_ok=True)
+        # dump config.json
+        cfg_fpath = os.path.join(path, "config.json")
+        if logger:
+            logger.debug(f"Creating machine descriptor file '{cfg_fpath}'")
+        with open(cfg_fpath, "wt") as fout:
+            json.dump({
+                "version": "1.0",
+                "type": self.type,
+                "description": "",
+                "configuration": self._configuration
+            }, fout, indent=4, sort_keys=True)
+        os.chmod(cfg_fpath, 0o600)
+
+    def remove(self, logger: Optional[logging.Logger] = None):
+        from . import cpkconfig
+        path = os.path.join(cpkconfig.path, "machines", self.name)
+        if logger:
+            logger.debug(f"Removing machine directory '{path}'")
+        if os.path.exists(path):
+            rmtree(path)
+
+    def get_architecture(self) -> str:
+        if self._arch is not None:
+            return self._arch
+        # get client
+        client = self.get_client()
+        try:
+            # fetch architecture from client
+            endpoint_arch = client.info()["Architecture"]
+            if endpoint_arch not in CANONICAL_ARCH:
+                raise CPKException(f"Unsupported architecture '{endpoint_arch}'.")
+        finally:
+            client.close()
+        return CANONICAL_ARCH[endpoint_arch]
+
+    def get_ncpus(self, ) -> Optional[int]:
+        # noinspection PyBroadException
+        try:
+            return self.get_client().info()["NCPU"]
+        except BaseException:
+            return None
+
+    def login_client(self):
+        username = os.environ.get('DOCKER_USERNAME', None)
+        password = os.environ.get('DOCKER_PASSWORD', None)
+        if username is not None and password is not None:
+            self.get_client().login(username=username, password=password)
+
+    def pull_image(self, image, progress=True) -> bool:
+        client = self.get_client()
+        layers = set()
+        pulled = set()
+        pbar = ProgressBar() if progress else None
+        for line in client.api.pull(image, stream=True, decode=True):
+            if "id" not in line or "status" not in line:
+                continue
+            layer_id = line["id"]
+            layers.add(layer_id)
+            if line["status"] in ["Already exists", "Pull complete"]:
+                pulled.add(layer_id)
+            # update progress bar
+            if progress:
+                percentage = max(0.0, min(1.0, len(pulled) / max(1.0, len(layers)))) * 100.0
+                pbar.update(percentage)
+        if progress:
+            pbar.done()
+        return True
+
+    def push_image(self, image, progress=True, **kwargs):
+        client = self.get_client()
+        layers = set()
+        pushed = set()
+        pbar = ProgressBar() if progress else None
+        for line in client.api.push(*image.split(":"), stream=True, decode=True, **kwargs):
+            if "id" not in line or "status" not in line:
+                continue
+            layer_id = line["id"]
+            layers.add(layer_id)
+            if line["status"] in ["Layer already exists", "Pushed"]:
+                pushed.add(layer_id)
+            # update progress bar
+            if progress:
+                percentage = max(0.0, min(1.0, len(pushed) / max(1.0, len(layers)))) * 100.0
+                pbar.update(percentage)
+        if progress:
+            pbar.done()
+
+    def __str__(self):
+        return """
+Machine:
+    Type:\t{}
+    Name:\t{}
+    URL:\t{}
+""".format(type(self).__name__, self.name, self.base_url)
+
+
+@dataclasses.dataclass
+class CPKConfiguration:
+    path: str
+    machines: Dict[str, Machine]
